# -*- coding: utf-8 -*-
"default plugin for setting: set it in a simple dictionary"
# Copyright (C) 2013 Team tiramisu (see AUTHORS for all contributors)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# ____________________________________________________________
from tiramisu.plugins.sqlite3.cache import PluginCache


class PluginSettings(PluginCache):
    __slots__ = tuple()

    def __init__(self, config_id):
        settings_table = 'CREATE TABLE IF NOT EXISTS property(path text primary key, properties text)'
        permissives_table = 'CREATE TABLE IF NOT EXISTS permissive(path text primary key, permissives text)'
        # should init cache too
        super(PluginSettings, self).__init__(config_id, 'property')
        self._cursor.execute(settings_table)
        self._cursor.execute(permissives_table)
        self._conn.commit()
    # sqlite

    def _p_sqlite_getpath(self, opt):
        if opt is None:
            return '_none'
        else:
            return self._get_opt_path(opt)

    # propertives
    def _p_setproperties(self, opt, properties):
        path = self._p_sqlite_getpath(opt)
        self._cursor.execute("DELETE FROM property WHERE path = ?", (path,))
        self._cursor.execute("INSERT INTO property(path, properties) VALUES (?, ?)",
                             (path, self._p_sqlite_encode(properties)))
        self._conn.commit()

    def _p_getproperties(self, opt, default_properties):
        path = self._p_sqlite_getpath(opt)
        self._cursor.execute("SELECT properties FROM property WHERE path = ?", (path,))
        value = self._cursor.fetchone()
        if value is None:
            return set(default_properties)
        else:
            return set(self._p_sqlite_decode(value[0]))

    def _p_hasproperties(self, opt):
        path = self._p_sqlite_getpath(opt)
        return self._cursor.execute("SELECT properties FROM property WHERE path = ?", (path,)) is not None

    def _p_reset_all_propertives(self):
        self._cursor.execute("DELETE FROM property")
        self._conn.commit()

    def _p_reset_properties(self, opt):
        path = self._p_sqlite_getpath(opt)
        self._cursor.execute("DELETE FROM property WHERE path = ?", (path,))
        self._conn.commit()

    def _p_get_properties(self):
        """return all properties in a dictionary
        """
        self._cursor.execute("SELECT * FROM property")
        ret = {}
        for path, properties in self._cursor.fetchall():
            if path == '_none':
                opt = None
            else:
                opt = self.context.cfgimpl_get_description().impl_get_opt_by_path(path)
            properties = self._p_sqlite_decode(properties)
            ret[opt] = properties
        return ret

    # permissive
    def _p_setpermissive(self, opt, permissive):
        path = self._p_sqlite_getpath(opt)
        self._cursor.execute("DELETE FROM permissive WHERE path = ?", (path,))
        self._cursor.execute("INSERT INTO permissive(path, permissives) VALUES (?, ?)",
                             (path, self._p_sqlite_encode(permissive)))
        self._conn.commit()

    def _p_getpermissive(self, opt=None):
        path = self._p_sqlite_getpath(opt)
        self._cursor.execute("SELECT permissives FROM permissive WHERE path = ?", (path,))
        permissives = self._cursor.fetchone()
        if permissives is None:
            return frozenset()
        else:
            return frozenset(self._p_sqlite_decode(permissives[0]))
